# -*- coding: utf-8 -*-

from rummy.deck.deck import Deck
from rummy.player.dealer import Dealer


class Round(Dealer):
    first_player = 0
    current_player = 0
    turn = 1
    last_turn = 1
    knocked = False

    def __init__(self, players):
        super().__init__()
        self.players = players
        self.deck = Deck()

    def prepare_new_round(self):
        self.turn = 1
        self.last_turn = 1
        self.knocked = False
        self.deck = Deck()

    def deal_cards(self, players):
        for p in players:
            p.hand = self.deal(self.deck.get_deck())

    def prepare_turn(self):
        self.deck.check_stack()
        self.check_knocked()

    def get_turn(self, round):
        # Todo: find out why round is being passed back to itself
        return "Turn %i, %s\n" % (round.turn, round.get_current_players_name())

    def get_current_players_hand(self):
        return self.players[self.current_player].get_hand_to_print()

    def end_turn(self):
        self.switch_current_player()
        self.turn += 1

    def check_knocked(self):
        if self.knocked:
            self.last_turn += 1

    def switch_current_player(self):
        self.current_player = (self.current_player + 1) % len(self.players)

    def rotate_first_player(self):
        self.first_player += 1
        self.current_player = self.first_player % len(self.players)



# -*- coding: utf-8 -*-

from rummy.deck.card import Card
from rummy.deck.face_values import FaceValues
from rummy.deck.suits import Suits


class Rank:

    def __init__(self):
        self.ranked_cards = [Card(value, suit) for suit in Suits.get() for value in FaceValues.get()]

    def __repr__(self):
        return str([(i, str(card)) for i, card in enumerate(self.ranked_cards)])

    def get_suit_and_rank_key(self, card):
        return Suits.get().index(card.suit), FaceValues.get().index(card.value)

    def get_rank_key(self, card):
        return FaceValues.get().index(card.value)


if __name__ == '__main__':
    print(Rank().ranked_cards)



# -*- coding: utf-8 -*-
from random import choice
from time import sleep

from text_template import TextTemplate as View

from rummy.constants.constants import TEMPLATE_PATH
from rummy.player.player import Player


class AI(Player):

    # Todo remove all prints return only data create a new class to handle displaying data
    def choose_to_discard_or_pick_up(self):
        if self.round.deck.has_discard():
            if self.ai_only:
                self.render_player_turn_start()
                self.ai_thinking('Choosing pick up')
            else:
                self.render_ai_turn_start()
            self.choose_pick_up()
        else:
            if self.ai_only:
                self.render_player_turn_start()
            else:
                self.render_ai_turn_start()
            self.ai_thinking('Drawing first card from deck')
            self.hand.draw_card(self.round.deck.take_card())

    def choose_pick_up(self):
        current_score = self.hand.get_score()
        if self.ai_only:
            print('Current Score: ', current_score)
        scores = self.melds.find_discard_scores(self.hand.get_hand(), self.round.deck.inspect_discard())
        if self.ai_only:
            print('Possible Hand Scores: ', scores)
            print('Min Score: ', min(scores))
        if min(scores) < current_score - 4 or min(scores) <= 10:
            self.ai_thinking('Drawing from discard')
            self.hand.draw_card(self.round.deck.take_discard())
        else:
            self.ai_thinking('Drawing from deck')
            self.hand.draw_card(self.round.deck.take_card())

    def discard_or_knock(self):
        if self.ai_only:
            self.render_ai_turn_end()
        self.ai_thinking('Choosing card to discard')
        scores = self.melds.find_discard_scores(self.hand.get_hand())
        score = min(scores)
        if score <= 10 and not self.round.knocked:
            self.round.knocked = True
        if scores.count(score) > 1:
            choices = [(i, x) for (i, x) in enumerate(scores) if (x == score)]
            discard = choice(choices)[0]
        else:
            discard = scores.index(score)
        discard = self.hand.discard_card(discard)
        if self.ai_only:
            print('Discarding: ', discard)
            print('Hand Score: ', score)
        self.round.deck.discard_card(discard)

    # Todo: move to new view class
    def ai_thinking(self, action):
        print(View.render(
            template=TEMPLATE_PATH + '/ai-thinking.txt',
            action=action,
        ))
        if not self.ai_only:
            sleep(0.8)

    def render_ai_turn_end(self):
        print(View.render(
            template=TEMPLATE_PATH + '/ai-turn-end.txt',
            hand=str(self.hand),
        ))



# -*- coding: utf-8 -*-

from abc import ABCMeta, abstractmethod

from text_template import TextTemplate as View

from game.melds import Melds
from rummy.constants.constants import TEMPLATE_PATH
from rummy.player.hand import Hand


class Player(metaclass=ABCMeta):
    __metaclass__ = ABCMeta

    def __init__(self, num):
        self.num = num
        self.game_score = 0
        self.hand = Hand()
        self.melds = Melds()
        self.round = None
        self.ai_only = False

    def __str__(self):
        return "Player %i" % self.num

    def turn(self, game_round, ai_only):
        self.round = game_round
        self.ai_only = ai_only
        self.has_someone_knocked()
        self.choose_to_discard_or_pick_up()
        self.discard_or_knock()

    def get_name(self):
        return "Player %i" % self.num

    def update_score(self):
        self.game_score += self.hand.get_score()

    def get_hand(self):
        return self.hand

    def get_game_score(self):
        return self.game_score

    def display_round_score(self):
        return self.hand.score

    def has_someone_knocked(self):
        if self.round.knocked:
            print(View.render(template=TEMPLATE_PATH + '/knocked.txt'))

    # Todo: move to new view class
    def render_player_turn_start(self):
        print(View.render(
            template=TEMPLATE_PATH + '/player-turn-start.txt',
            turn_number=self.round.turn,
            player_number=self.round.current_player + 1,
            score=self.hand.get_score(),
            hand=str(self.hand),
            discard=self.round.deck.show_discard()
        ))

    def render_ai_turn_start(self):
        print(View.render(
            template=TEMPLATE_PATH + '/turn-start.txt',
            turn_number=self.round.turn,
            player_number=self.round.current_player + 1,
            discard=self.round.deck.show_discard()
        ))

    def render_player_turn_end(self):
        print(View.render(
            template=TEMPLATE_PATH + '/player-turn-end.txt',
            hand=str(self.hand),
            key=self.hand.get_key()
        ))

    @abstractmethod
    def choose_to_discard_or_pick_up(self):
        pass

    @abstractmethod
    def discard_or_knock(self):
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from time import sleep

from ansi_colours import AnsiColours as Colour

from console_config import ConsoleConfig
from rummy.game.round import Round
from rummy.game.score import Score
from rummy.game.setup_players import SetupPlayers
from rummy.player.human import Human


class Play():
    def __init__(self):
        ConsoleConfig.colorama()
        setup = SetupPlayers()
        self.players = setup.create_players()
        self.ai_only = not any(isinstance(x, Human) for x in self.players)
        self.score = Score(self.players)
        self.round = Round(self.players)
        self.round.deal_cards(self.players)
        self.play_game()

    def play_game(self):
        while self.round.last_turn != len(self.players):
            self.round.prepare_turn()
            player = self.players[self.round.current_player]
            player.turn(self.round, self.ai_only)
            self.round.end_turn()
        self.end_round()
        sleep(1.2)
        self.start_new_round_or_end_game()

    def start_new_round_or_end_game(self):
        if self.score.is_end_of_game():
            self.score.end_game()
        else:
            self.round.rotate_first_player()
            if not self.ai_only:
                self.confirm_start_new_round()
            self.round.prepare_new_round()
            self.round.deal_cards(self.players)
            self.play_game()

    def confirm_start_new_round(self):
        print("\nReady %s?" % self.players[self.round.current_player])
        ready = ''
        while ready.lower() != 'y':
            ready = input("Enter " + Colour.green('y') + " when you are ready for the next round: ")

    def end_round(self):
        self.score.update_player_scores()
        self.score.display_this_round_score()


# start game
if __name__ == "__main__":
    Play()



# -*- coding: utf-8 -*-

from rummy.player.hand import Hand


class Dealer:
    cardCount = 7

    def set_card_count(self, card_count: int):
        self.cardCount = card_count

    def deal(self, deck: list):
        hand = Hand()
        for _ in range(self.cardCount):
            hand.draw_card(deck.pop())
        return hand



# -*- coding: utf-8 -*-
from time import sleep

from ansi_colours import AnsiColours as Colour
from text_template import TextTemplate as View

from rummy.constants.constants import TEMPLATE_PATH


class Score:
    def __init__(self, players):
        self.players = players

    def display_this_round_score(self):
        print(View.render(
            template=TEMPLATE_PATH + '/round-end.txt',
            round_scores=self.get_end_of_round_scores(),
            game_scores=self.get_current_game_scores()
        ))

    def get_current_game_scores(self):
        return ''.join(["%s: %s\n" % (p, p.get_game_score()) for p in self.players])

    def get_end_of_round_scores(self):
        output = ''
        less_than_ten = False
        for p in self.players:
            score = p.hand.get_score()
            if score <= 10:
                less_than_ten = True
            output += View.render(
                template=TEMPLATE_PATH + '/hand-score.txt',
                player=p.get_name(),
                hand=str(p.hand),
                score=score
            )
        if not less_than_ten:
            print(Colour.red('<+++++++++++++++++++BUG+++++++++++++++++++++++++'))
            sleep(8)
        return output

    def update_player_scores(self):
        for p in self.players:
            p.hand.get_score()
            p.update_score()

    def is_end_of_game(self):
        for p in self.players:
            if p.get_game_score() >= 100:
                return True
        return False

    def end_game(self):
        winners = self.find_lowest_scores()
        if len(winners) == 1:
            print(Colour.green("%s is the Winner!!" % winners[0]))
        else:
            print(Colour.green(", ".join([w for w in winners]) + " are joint winners!"))

    def find_lowest_scores(self):
        lowest = []
        for p in self.players:
            if not lowest:
                lowest = [p]
                continue
            if p.get_game_score() < lowest[0].get_game_score():
                lowest = [p]
            elif p.get_game_score() == lowest[0].get_game_score():
                lowest.append(p)
        return lowest



# -*- coding: utf-8 -*-

from rummy.player.ai import AI
from rummy.player.human import Human


class SetupPlayers:
    players = []
    number_of_players = -1
    number_of_opponents = -1

    def __init__(self):
        self.choose_players()

    def choose_players(self):
        number_of_players = -1
        while number_of_players not in [i for i in range(0, 5)]:
            number_of_players = input("Enter number of players (0-4)? ")
            number_of_players = self.valid_number_check(number_of_players)
        if number_of_players in [0, 1]:
            self.setup_ai(number_of_players)
        self.number_of_players = number_of_players

    def setup_ai(self, number_of_players):
        self.ai = True
        if number_of_players == 0:
            self.choose_number_of_ai_opponents(4)
        elif number_of_players == 1:
            self.choose_number_of_ai_opponents(3)

    def choose_number_of_ai_opponents(self, max_opponents):
        number_of_opponents = -1
        while number_of_opponents not in [i for i in range(max_opponents - 2, max_opponents + 1)]:
            number_of_opponents = input(
                "Enter number of opponents ({0}-{1})? ".format(max_opponents - 2, max_opponents))
            number_of_opponents = self.valid_number_check(number_of_opponents)
        self.number_of_opponents = number_of_opponents

    # Todo: move to a new class
    @staticmethod
    def valid_number_check(number):
        try:
            number = int(number)
        except ValueError:
            number = -1
            print("Not a valid number, please try again...")
        return number

    def create_players(self):
        i = 0
        players = []
        for j in range(self.number_of_players):
            i += 1
            players.append(Human(i))
        for j in range(self.number_of_opponents):
            i += 1
            players.append(AI(i))
        return players



# -*- coding: utf-8 -*-

from ansi_colours import AnsiColours as Colour

from constants.constants import UNICODE_SUPPORT
from deck.suits import Suits


class Card:
    value = ""
    suit = ""

    def __init__(self, value, suit):
        self.value = value
        self.suit = Suits.alpha_to_unicode_suit_glyph(suit)

    def __str__(self):
        return "%s%s" % (self.value, self.suit)

    def __eq__(self, other):
        return self.value == other.value and self.suit == other.suit

    def get_card_colour(self):
        if self.suit in (u"♥", u"♦", "H", "D"):
            return self.red_card()
        elif self.suit in (u"♠", u"♣", "C", "S"):
            return self.black_card()

    def red_card(self):
        if UNICODE_SUPPORT:
            return str(self.value) + Colour.red(self.suit)
        else:
            return str(self.value) + self.suit

    def black_card(self):
        return str(self.value) + self.suit



# -*- coding: utf-8 -*-

from ansi_colours import AnsiColours as Colour

from game.melds import Melds
from rummy.deck.rank import Rank
from rummy.deck.sort import Sort


class Hand(Rank):

    def __init__(self):
        super().__init__()
        self.colour = Colour()
        self.hand = []
        self.score = 0
        self.sort = Sort()
        self.melds = Melds()

    def __str__(self):
        self.hand = self.sort.sort_hand_by_suit_and_rank(self.hand)
        return ', '.join([card.get_card_colour() for card in self.hand])

    def set_hand(self, hand):
        self.hand = hand

    def get_hand(self):
        return self.hand

    def draw_card(self, card):
        self.hand.append(card)

    def discard_card(self, choice):
        return self.hand.pop(choice)

    def get_score(self):
        return self.melds.calculate_score(self.hand)

    def get_key(self):
        return ',  '.join(["%s" % Colour.green(str((i + 1))) for i in range(len(self.hand))])


z



# -*- coding: utf-8 -*-

from ansi_colours import AnsiColours as Colour

from rummy.player.player import Player


class Human(Player):

    # Todo separate prints and inputs from methods and move to a new class, methods should return data
    def choose_to_discard_or_pick_up(self):
        self.render_player_turn_start()
        if self.round.deck.has_discard():
            self.choose_pick_up()
        else:
            self.hand.draw_card(self.round.deck.take_card())

    def choose_pick_up(self):
        player_choice = self.get_user_pick_up_input()
        if player_choice == 'p':
            self.hand.draw_card(self.round.deck.take_discard())
        else:
            self.hand.draw_card(self.round.deck.take_card())

    def discard_or_knock(self):
        self.render_player_turn_end()
        scores = self.melds.find_discard_scores(self.hand.get_hand())
        if min(scores) <= 10 and not self.round.knocked:
            message = "Enter a number to discard a card or " + Colour.green('k') + " to Knock: "
        else:
            message = "Enter a number to discard a card: "
        player_choice = ""
        while player_choice not in [str(i) for i in range(1, 9)]:
            if self.round.knocked:
                message = "Enter a number to discard a card: "
            player_choice = input(message)
            if player_choice.lower() == "k" and min(scores) < 10:
                self.round.knocked = True
        player_choice = int(player_choice) - 1
        self.round.deck.discard_card(self.hand.discard_card(player_choice))

    # Todo: move to new UI class
    def get_user_pick_up_input(self):
        player_choice = ''
        while player_choice.lower() not in ['d', 'p']:
            player_choice = input(
                "Enter " + Colour.green('d') + " to draw or " + Colour.green('p') + " to pickup discard: ")
        return player_choice



import os

import colorama


class ConsoleConfig:
    # Todo: move this somewhere else
    @staticmethod
    def colorama():
        if 'PYCHARM_HOSTED' in os.environ:
            convert = False  # in PyCharm, we should disable convert
            strip = False
        else:
            convert = None
            strip = None
        colorama.init(convert=convert, strip=strip)



# -*- coding: utf-8 -*-

from random import shuffle

from rummy.deck.rank import Rank


class Deck(Rank):
    deck = []
    discard_pile = []

    def __init__(self):
        super().__init__()
        self.deck = self.ranked_cards[:]
        self.discard_pile = []
        shuffle(self.deck)

    def check_stack(self):
        if len(self.deck) == 0:
            self.deck = self.discard_pile
            self.discard_pile = []

    def show_discard(self):
        return self.discard_pile[-1].get_card_colour() if len(self.discard_pile) else 'Empty'

    def get_deck(self):
        return self.deck

    def get_discard_pile(self):
        return self.discard_pile

    def has_discard(self):
        return len(self.discard_pile) > 0

    def take_card(self):
        return self.deck.pop()

    def take_discard(self):
        return self.discard_pile.pop()

    def inspect_discard(self):
        return self.discard_pile[-1]

    def discard_card(self, card):
        return self.discard_pile.append(card)



# coding=utf-8
import sys

import pkg_resources

TEMPLATE_PATH = pkg_resources.resource_filename('rummy', 'templates/')
UNICODE_SUPPORT = sys.stdout.encoding.lower().startswith('utf')
VALUES = ('A', '2', '3', '4', '5', '6', '7', '8', '9', 'T', 'J', 'Q', 'K')
SUITS = ("♠", "♥", "♦", "♣")
ALPHA_SUITS = ("S", "H", "D", "C")



# coding=utf-8
from constants.constants import VALUES


class FaceValues:

    @staticmethod
    def get() -> tuple:
        return VALUES

    @staticmethod
    def is_value_in_range(value):
        return value in VALUES



# coding=utf-8
from constants.constants import UNICODE_SUPPORT, ALPHA_SUITS, SUITS


class Suits:

    @staticmethod
    def get() -> tuple:
        return SUITS if UNICODE_SUPPORT else ALPHA_SUITS

    @staticmethod
    def alpha_to_unicode_suit_glyph(suit):
        if UNICODE_SUPPORT and suit in ALPHA_SUITS:
            suit = SUITS[ALPHA_SUITS.index(suit)]
        return suit



# coding=utf-8
from setuptools import setup, find_packages

readme = open('README.md', 'r')
README_TEXT = readme.read()
readme.close()

setup(
    name='rummy',
    version='1.1.0',
    url='https://github.com/sarcoma/Python-Rummy',
    license='MIT',
    author='sarcoma',
    author_email='sean@orderandchaoscreative.com',
    description='Console Rummy game',
    long_description=README_TEXT,
    long_description_content_type='text/markdown',
    entry_points={
        'console_scripts': ['rummy=rummy.command_line:main'],
    },
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data={'rummy':"templates/*.txt"},
    include_package_data=True,
    install_requires=['colorama', 'text_template', 'ansi_colours'],
    project_urls={
        'Order & Chaos Creative': 'https://orderandchaoscreative.com',
    },
    python_requires='>=3.5'
)



# -*- coding: utf-8 -*-

from rummy.deck.rank import Rank


class Sort:

    def __init__(self):
        self.rank = Rank()

    def sort_hand_by_suit_and_rank(self, hand):
        return sorted(hand, key=lambda card: self.rank.get_suit_and_rank_key(card))

    def sort_hand_by_rank(self, hand):
        return sorted(hand, key=lambda card: self.rank.get_rank_key(card))
